"""
CSSStyleSheet implements DOM Level 2 CSS CSSStyleSheet.

TODO:
    - insertRule HIERARCHY_REQUEST_ERR for all cases?
    - ownerRule and ownerNode
"""
__all__ = ['CSSStyleSheet']
__docformat__ = 'restructuredtext'
__version__ = '0.9a6'

import xml.dom

import cssutils.stylesheets
import cssutils.tokenize


class CSSStyleSheet(cssutils.stylesheets.StyleSheet):
    """
     The CSSStyleSheet interface is a concrete interface used to represent
     a CSS style sheet i.e., a style sheet whose content type is
     "text/css".

    Properties
    ==========
    cssRules: of type CSSRuleList, (DOM readonly)
        The list of all CSS rules contained within the style sheet. This
        includes both rule sets and at-rules.
    ownerRule: of type CSSRule, readonly
        If this style sheet comes from an @import rule, the ownerRule
        attribute will contain the CSSImportRule. In that case, the
        ownerNode attribute in the StyleSheet interface will be None. If
        the style sheet comes from an element or a processing instruction,
        the ownerRule attribute will be None and the ownerNode attribute
        will contain the Node.

    Inherits properties from stylesheet.StyleSheet

    Format
    ======
    stylesheet
      : [ CHARSET_SYM S* STRING S* ';' ]?
        [S|CDO|CDC]* [ import [S|CDO|CDC]* ]*
        [ [ ruleset | media | page ] [S|CDO|CDC]* ]*            
    """
    type = 'text/css'
    
    def __init__(self, href=None, media=None,
                 title=u'', disabled=None,
                 ownerNode=None, parentStyleSheet=None,
                 readonly=False):
        
        super(CSSStyleSheet, self).__init__(
                self.type, href, media, title, disabled,
                ownerNode, parentStyleSheet)
        self.cssRules = cssutils.css.CSSRuleList()
        self._readonly = readonly


    def _getCssText(self):
        return cssutils.ser.do_stylesheet(self)
    
    def _setCssText(self, cssText):
        """
        (cssutils)
        Parses cssText and overwrites the whole stylesheet.

        cssText
            textual text to set

        DOMException on setting  
        
        - NO_MODIFICATION_ALLOWED_ERR: (self)
          Raised if the rule is readonly.
        - SYNTAX_ERR:
          Raised if the specified CSS string value has a syntax error and
          is unparsable.
        """
        self._checkReadonly()
        tokenizer = cssutils.tokenize.Tokenizer()
        ttypes, log = tokenizer.ttypes, tokenizer.log
        tokens = tokenizer.tokenize(cssText)
        
        newcssRules = cssutils.css.CSSRuleList()

        # @charset only once at beginning
        if tokens and tokens[0].type == ttypes.CHARSET_SYM: 
            charsetruletokens, endi = tokenizer.tokensupto(tokens)
            charsetrule = cssutils.css.CSSCharsetRule()
            charsetrule.cssText = charsetruletokens
            newcssRules.append(charsetrule)
            i = endi + 1 # ?
        else:
            i = 0
        expected = '@import' # or any
        while i < len(tokens):
            t = tokens[i]
            
            # ignore
            if t.type in (ttypes.S, ttypes.CDO, ttypes.CDC): 
                pass

            # unexpected: } ;
            elif t.type in (ttypes.SEMICOLON,
                            ttypes.LBRACE, ttypes.RBRACE): 
                log.error(u'CSSStyleSheet: Syntax Error',t)

            # simply add
            elif ttypes.COMMENT == t.type: 
                newcssRules.append(cssutils.css.CSSComment(t))
                
            # @charset only at beginning
            elif ttypes.CHARSET_SYM == t.type: 
                atruletokens, endi = tokenizer.tokensupto(tokens[i:])
                i += endi 
                log.error(u'CSSStylesheet: CSSCharsetRule only allowed at beginning.',
                    t, xml.dom.HierarchyRequestErr)

            # @import before any StyleRule and @rule except @charset
            elif ttypes.IMPORT_SYM == t.type:
                atruletokens, endi = tokenizer.tokensupto(tokens[i:])
                if expected == '@import': 
                    atrule = cssutils.css.CSSImportRule()
                    atrule.cssText = atruletokens
                    newcssRules.append(atrule)
                else:
                    log.error(
                        u'CSSStylesheet: CSSImportRule not allowed here.',
                        t, xml.dom.HierarchyRequestErr)
                i += endi 

            # @media
            elif ttypes.MEDIA_SYM == t.type:
                atruletokens, endi = tokenizer.tokensupto(tokens[i:])
                atrule = cssutils.css.CSSMediaRule()
                atrule.cssText = atruletokens
                newcssRules.append(atrule)
                i += endi 
                expected = 'any'

            # @page
            # @namespace

            # @unknown
            elif ttypes.ATKEYWORD == t.type:
                atruletokens, endi = tokenizer.tokensupto(tokens[i:])
                atrule = cssutils.css.CSSUnknownRule()
                atrule.cssText = atruletokens
                newcssRules.append(atrule)
                i += endi 

            else: # probable StyleRule
                ruletokens, endi = tokenizer.tokensupto(
                    tokens[i:], blockendonly=True)
                rule = cssutils.css.CSSStyleRule()
                rule.cssText = ruletokens
                newcssRules.append(rule)
                i += endi                   
                expected = 'any'

            i += 1

        self.cssRules = newcssRules

    cssText = property(_getCssText, _setCssText,
            "(cssutils) a textual representation of the stylesheet")


    def deleteRule(self, index):
        """
        Used to delete a rule from the style sheet.

        index
            of the rule to remove in the StyleSheet's rule list  

        DOMException
    
        - INDEX_SIZE_ERR: (self)
          Raised if the specified index does not correspond to a rule in
          the style sheet's rule list.
        - NO_MODIFICATION_ALLOWED_ERR: (self)
          Raised if this style sheet is readonly.
        """
        self._checkReadonly()
        
        try:
            del self.cssRules[index]
        except IndexError:
            raise xml.dom.IndexSizeErr(
                u'CSSStyleSheet: %s is not a valid index in the rulelist of length %i' % (
                index, self.cssRules.length))


    def insertRule(self, rule, index=None):
        """
        Used to insert a new rule into the style sheet. The new rule now
        becomes part of the cascade.

        Rule may be a string or a valid CSSRule.        

        rule
            a parsable DOMString (cssutils: or Rule object)
        index
            of the rule before the new rule will be inserted.
            If the specified index is equal to the length of the
            StyleSheet's rule collection, the rule will be added to the end
            of the style sheet.
            If index is not given or None rule will be appended to rule
            list.
            
        returns the index within the stylesheet's rule collection

        DOMException
        
        - HIERARCHY_REQUEST_ERR: (self)
          Raised if the rule cannot be inserted at the specified index
          e.g. if an @import rule is inserted after a standard rule set
          or other at-rule.
        - INDEX_SIZE_ERR: (not raised at all)
          Raised if the specified index is not a valid insertion point.
        - NO_MODIFICATION_ALLOWED_ERR: (self)
          Raised if this style sheet is readonly.
        - SYNTAX_ERR: (rule)
          Raised if the specified rule has a syntax error and is
          unparsable.        
        """
        self._checkReadonly()
        
        tokenizer = cssutils.tokenize.Tokenizer()
        log = tokenizer.log

        # check position
        if index is None:
            index = len(self.cssRules)
        elif index < 0 or index > self.cssRules.length:
            raise xml.dom.IndexSizeErr(
                u'CSSStyleSheet: Invalid index %s for CSSRuleList with a length of %s.' % (
                    index, self.cssRules.length))
        
        # parse
        if isinstance(rule, basestring):
            tempsheet = CSSStyleSheet()
            tempsheet.cssText = rule
            if len(tempsheet.cssRules) != 1 or (tempsheet.cssRules and
             not isinstance(tempsheet.cssRules[0], cssutils.css.CSSRule)):
                log.error(u'CSSStyleSheet: Invalid Rule: %s' % rule)
                return
            rule = tempsheet.cssRules[0]
        elif not isinstance(rule, cssutils.css.CSSRule):
            log.error(u'CSSStyleSheet: Not a CSSRule: %s' % rule)
            return

        # CHECK HIERARCHY
        # @charset
        if isinstance(rule, cssutils.css.CSSCharsetRule):
            if index != 0 or (self.cssRules and
             isinstance(self.cssRules[0], cssutils.css.CSSCharsetRule)):
                log.error(u'CSSStylesheet: @charset only once allowed at the beginning.',
                      error=xml.dom.HierarchyRequestErr)
                return
            else:
                self.cssRules.insert(index, rule)
            
        # @unknown or comment
        elif isinstance(rule, cssutils.css.CSSUnknownRule) or \
           isinstance(rule, cssutils.css.CSSComment):
            if index == 0 and self.cssRules and \
             isinstance(self.cssRules[0], cssutils.css.CSSCharsetRule):
                log.error(u'CSSStylesheet: @charset must be the first rule.',
                      error=xml.dom.HierarchyRequestErr)
                return
            else:
                self.cssRules.insert(index, rule)

        # @import
        elif isinstance(rule, cssutils.css.CSSImportRule):
            # after @charset
            if index == 0 and self.cssRules and \
               isinstance(self.cssRules[0], cssutils.css.CSSCharsetRule):
                log.error(u'CSSStylesheet: @import must be after @charset.',
                      error=xml.dom.HierarchyRequestErr)
                return
            # before @media and stylerule
            for r in self.cssRules[:index]:
                if isinstance(r, cssutils.css.CSSMediaRule) or \
                   isinstance(r, cssutils.css.CSSStyleRule):
                    log.error(u'CSSStylesheet: Only allowed before @import is @charset and CSSUnknownRule.',
                      error=xml.dom.HierarchyRequestErr)
                    return
            self.cssRules.insert(index, rule) 

        # all other
        else:
            for r in self.cssRules[index:]:
                if isinstance(r, cssutils.css.CSSCharsetRule) or \
                   isinstance(r, cssutils.css.CSSImportRule):
                    log.error(u'CSSStylesheet: Only allowed after @charset and @import.',
                      error=xml.dom.HierarchyRequestErr)
                    return
            self.cssRules.insert(index, rule)

        return index

            
    def addRule(self, rule):
        "DEPRECATED, use appendRule instead"
        import warnings
        warnings.warn(
            '``addRule`` is deprecated: Use ``insertRule(rule)``.',
            DeprecationWarning)
        return self.insertRule(rule)


    def _getsetOwnerRuleDummy(self):
        """
        NOT IMPLEMENTED YET
        If this style sheet comes from an @import rule, the ownerRule
        attribute will contain the CSSImportRule. In that case, the
        ownerNode attribute in the StyleSheet interface will be null. If
        the style sheet comes from an element or a processing instruction,
        the ownerRule attribute will be null and the ownerNode attribute
        will contain the Node.
        """
        raise NotImplementedError()

    ownerRule = property(_getsetOwnerRuleDummy, _getsetOwnerRuleDummy,
                         doc="(DOM attribute) NOT IMPLEMENTED YET")


    def setSerializer(self, cssserializer):
        """
        Sets Serializer used for output of this stylesheet
        """
        if isinstance(cssserializer, cssutils.CSSSerializer):
            cssutils.ser = cssserializer
        else:
            raise ValueError(u'Serializer must be an instance of cssutils.CSSSerializer.')

    def setSerializerPref(self, pref, value):
        """
        Sets Preference of CSSSerializer used for output of this
        stylesheet. See cssutils.serialize.Preferences for possible
        preferences to be set.
        """
        cssutils.ser.prefs.__setattr__(pref, value)


if __name__ == '__main__':
    print "CSSStyleSheet"
    c = CSSStyleSheet(href=None,
                        #
                        media="all",
                        title="test",
                        disabled=None,
                        ownerNode=None,
                        parentStyleSheet=None)
    c.cssText = u'@charset "utf-8";'
    print "title: ", c.title
    print "type: ", c.type
    print "cssText: ", c.cssText
    print c.cssRules
    c.addRule(cssutils.css.CSSImportRule('x'))

    